/*******************************************************************************
*
* FILE:         aesapi.h
*
* DESCRIPTION:  Header file for aesapi.c
*
* AUTHOR:       Chae Hoon Lim and Hyo Sun Hwang
*               E-mail: chlim@future.co.kr, hyosun@future.co.kr
*               http://crypt.future.co.kr/~chlim
*               Information and Communications Research Center,
*               Future Systems, Inc.
*               372-2, Yang Jae-Dong, Seo Cho-Gu, Seoul, 137-130, KOREA
*               Tel: +82-2-578-0581 (ext. 557), Fax: +82-2-578-0584
*
* DATE:         July 10, 1998
*
*******************************************************************************/

#ifndef AESAPI_H
#define AESAPI_H

/***************************** include filess *********************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "crypton.h"

/*************************** Constant Definitions ****************************/

/* Defines: Add any additional defines you need */
#define DIR_ENCRYPT     0   /* Are we encrpyting? */
#define DIR_DECRYPT     1   /* Are we decrpyting? */
#define MODE_ECB        1   /* Are we ciphering in ECB mode? */
#define MODE_CBC        2   /* Are we ciphering in CBC mode? */
#define MODE_CFB1       3   /* Are we ciphering in 1-bit CFB mode? */
#define TRUE            1
#define FALSE           0

/* Error Codes - CHANGE POSSIBLE: inclusion of additional error codes */
#define BAD_KEY_DIR      -1 /* Key direction is invalid, e.g., unknown value */
#define BAD_KEY_MAT      -2 /* Key material not of correct length */
#define BAD_KEY_INSTANCE -3 /* Key passed is not valid */
#define BAD_CIPHER_MODE  -4 /* Params struct passed to cipherInit invalid */
#define BAD_CIPHER_STATE -5 /* Cipher in wrong state (e.g., not initialized) */
#define BAD_INPUT_LEN    -6 /* blockEncrypt/blockDecrypt : wrong input lenth */

/* CHANGE POSSIBLE:  inclusion of algorithm specific defines */
#define MAX_KEY_SIZE    32  /* # of ASCII char's needed to represent a key */
#define MIN_KEY_SIZE     8  /* minimal key size in bytes */
#define MAX_IV_SIZE     16  /* # bytes needed to represent an IV */
#define BLOCK_SIZE      16  /* in bytes */
#define BLOCK_LEN      128  /* in bits */ 

/************************* Type Definitions *********************************/

/* The structure for key information */
typedef struct {
    BYTE  direction;        /* Key used for encrypting or decrypting? */
    int   keyLen;           /* Length of the key  */
    char  keyMaterial[MAX_KEY_SIZE+1];
                    /* Raw key data in ASCII, e.g., user input or KAT values */

    DWORD KS[NoRoundKeys];  /* Enc/Decrypt Round Key Schedule */
    } keyInstance;

/* The structure for cipher information */
typedef struct {
    BYTE  mode;             /* MODE_ECB, MODE_CBC, or MODE_CFB1 */
    BYTE  IV[MAX_IV_SIZE];  /* A possible Initialization Vector for ciphering */
    } cipherInstance;

/************************* Prototype Declarations **************************/

int makeKey(keyInstance *key, BYTE direction, int keyLen, char *keyMaterial);

int cipherInit(cipherInstance *cipher, BYTE mode, char *IV);

int blockEncrypt(cipherInstance *cipher, keyInstance *key,
                    BYTE *input, int inputLen, BYTE *outBuffer);

int blockDecrypt(cipherInstance *cipher, keyInstance *key,
                    BYTE *input, int inputLen, BYTE *outBuffer);

/******************************************************************************/
#endif
