/*******************************************************************************
*
* FILE:         aestest.c
*
* DESCRIPTION:  main routine to generate the AES Test Data for CRYPTON Version 0.5
*
* AUTHOR:       Chae Hoon Lim and Hyo Sun Hwang
*               E-mail: chlim@future.co.kr, hyosun@future.co.kr
*               http://crypt.future.co.kr/~chlim
*               Information and Communications Research Center,
*               Future Systems, Inc.
*               372-2, Yang Jae-Dong, Seo Cho-Gu, Seoul, 137-130, KOREA
*               Tel: +82-2-578-0581 (ext. 557), Fax: +82-2-578-0584
*
* DATE:         July 10, 1998
*
* NOTE: This code contains various routines to generate the KAT and MCT data 
*       required in AES submission.This code must be compiled and linked with 
*       "crypton.o" and "aesapi.o" to produce the exe file "aestest.exe".
*
*******************************************************************************/

#include <time.h>
#include "aesapi.h"

#if !defined(CheckInterValue)
    #error FATAL ERROR :: must define 'CheckInterValue'
#endif

/******************** Definitiona and Global variables *********************/

#define MCT_ITER        10000
#define MaxIter4IntVal  128

#define FileHead(fp, filename, desc1, desc2) fprintf(fp, "\
\n\
=========================\n\
\n\
FILENAME:  \"%s\"\n\
\n\
%s\n\
%s\n\
\n\
Algorithm Name: Crypton\n\
Principal Submitter: Chae Hoon Lim\n\
\n\
==========\n", filename, desc1, desc2);

/* Print out message 'msg' and 'Data' to the file 'FP' */
#define BytePrint(FP, msg, DataSize, Data) {                \
    DWORD idx;                                              \
    fprintf(FP, "%s=", msg);                                \
    for( idx=0; idx<(DWORD)DataSize; idx++) {               \
        fprintf(FP, "%.2X", ((BYTE *)Data)[idx]);           \
    }                                                       \
}

/*  */
#define TEST(str, oper) {   \
    clock_t start, finish;  \
    start = clock();        \
    oper;                   \
    finish = clock();       \
    printf("\t%s : %fsec\n", str, (double)(finish-start)/ CLOCKS_PER_SEC);\
}


/* Global variables */
int PRN=0;
BYTE pdwInterVal[NoRounds+1][16];
BYTE Table_S0[MaxIter4IntVal][256], Table_S1[MaxIter4IntVal][256];
    /* These 4 variables are used in intermediate KAT tests */


/************************************************************************/
int main(argc, argv)
int argc;
char *argv[];
{
    void ecb_vk(), ecb_vt(), ecb_tbl();
    void ecb_e_m(), ecb_d_m(), cbc_e_m(), cbc_d_m();

#if defined(USE_ASM)
    printf("Using Inline Assemly");
#else
    printf("UsingC only");
#endif

#if defined(BIG_ENDIAN)
    printf(", BIG_ENDIAN defined.");
#else
    printf(", LITTLE_ENDIAN defined.");
#endif

    printf("\n");

    PRN = 1;	/* Save the intermediate values to pdwInterVal[][] */
    TEST("ecb_vk(), ecb_vk_int()", ecb_vk());
    TEST("ecb_vt(), ecb_vt_int()", ecb_vt());
    TEST("ecb_tbl", ecb_tbl());

    PRN = 0;
    TEST("ecb_e_m", ecb_e_m());
    TEST("ecb_d_m", ecb_d_m());
    TEST("cbc_e_m", cbc_e_m());
    TEST("cbc_d_m", cbc_d_m());

    return 1;
}


/*********************** Variable Key KAT in ECB mode ************************/
/*-------------- tests both the final and intermediate values--------------- */
void ecb_vk()
{
    FILE *fp1, *fp2;
    int i, j, k, KeyLen, outLen;
    BYTE UK[MAX_KEY_SIZE],  /* User Key */
         PT[BLOCK_SIZE], CT[BLOCK_SIZE], DT[BLOCK_SIZE];
    BYTE IM[10] = "\nI.M.??";
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("ecb_vk.txt...\n");
    fp1 = fopen("ecb_vk.txt", "w");
    fp2 = fopen("ecb_vk_int.txt", "w");
    FileHead(fp1, "ecb_vk.txt", "Electronic Codebook (ECB) Mode",
                    "Variable Key Known Answer Tests");
    FileHead(fp2, "ecb_vk_int.txt", "Electronic Codebook (ECB) Mode",
                    "Variable Key Known Answer Tests - Intermediate Values");
    for( i=0; i<BLOCK_SIZE; i++)
        PT[i] = 0;

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        fprintf(fp1, "\nKEYSIZE=%d\n", KeyLen);
        BytePrint(fp1, "\nPT", BLOCK_SIZE, PT);
        fprintf(fp2, "\nKEYSIZE=%d\n", KeyLen);
        BytePrint(fp2, "\nPT", BLOCK_SIZE, PT);

        for( i=0; i<KeyLen; i++) {
            /* Set UK : only i-th bit of UK be 1 */
            for( j=0; j<KeyLen/8; j++)  UK[j] = 0;
            UK[i>>3] = 1 << (7-(i&7));

            /* Encrypt process */
            makeKey(&key, DIR_ENCRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);
            outLen = blockEncrypt(&cipher, &key, PT, BLOCK_LEN, CT);
            if( outLen!=BLOCK_LEN ) {
                printf("Error in ecb_vk\n");
                exit(1);
            }

            /* print out i, UK and CT(final value) */
            fprintf(fp1, "\n\nI=%d", i+1);
            BytePrint(fp1, "\nKEY", KeyLen/8, UK);
            BytePrint(fp1, "\nCT", BLOCK_SIZE, CT);

            /* print out i, UK, CT and intermediate values */
            fprintf(fp2, "\n\nI=%d", i+1);
            BytePrint(fp2, "\nKEY", KeyLen/8, UK);
            BytePrint(fp2, "\nCT", BLOCK_SIZE, CT);
            for( k=0; k<=NoRounds; k++) {
                IM[5] = k/10 + '0';
                IM[6] = k%10 + '0';
                BytePrint(fp2, IM, BLOCK_SIZE, pdwInterVal[k]);
            }

            /* test that encryption and decryption are correct */
            makeKey(&key, DIR_DECRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);
            outLen = blockDecrypt(&cipher, &key, CT, BLOCK_LEN, DT);
            if( (outLen!=BLOCK_LEN) || (memcmp(PT, DT, BLOCK_SIZE)!=0) ) {
                printf("Error in ecb_vk\n");
                exit(1);
            }
        }
        fprintf(fp1, "\n\n==========\n");
        fprintf(fp2, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp1);
    fclose(fp2);
}

/********************** Variable Text KAT in ECB mode ********************/
/* ---------- tests both the intermediate and final values ------------- */
void ecb_vt()
{
    FILE *fp1, *fp2;
    int i, j, k, KeyLen, outLen;
    BYTE UK[MAX_KEY_SIZE],  /* User Key */
         PT[BLOCK_SIZE], CT[BLOCK_SIZE], DT[BLOCK_SIZE];
    BYTE IM[10] = "\nI.M.??";
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("ecb_vt.txt...\n");
    fp1 = fopen("ecb_vt.txt", "w");
    fp2 = fopen("ecb_vt_int.txt", "w");
    FileHead(fp1, "ecb_vt.txt", "Electronic Codebook (ECB) Mode",
                    "Variable Text Known Answer Tests");
    FileHead(fp2, "ecb_vt_int.txt", "Electronic Codebook (ECB) Mode",
                    "Variable Text Known Answer Tests - Intermediate Values");
    for( i=0; i<256/8; i++)
        UK[i] = 0;

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        fprintf(fp1, "\nKEYSIZE=%d\n", KeyLen);
        BytePrint(fp1, "\nKEY", KeyLen/8, UK);
        fprintf(fp2, "\nKEYSIZE=%d\n", KeyLen);
        BytePrint(fp2, "\nKEY", KeyLen/8, UK);

        for( i=0; i<BLOCK_LEN; i++) {
            /* Set PT : only i-th bit of PT be 1 */
            for( j=0; j<BLOCK_SIZE; j++)    PT[j] = 0;
            PT[i>>3] = 1 << (7-(i&7));

            /* Encrypt process */
            makeKey(&key, DIR_ENCRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);
            outLen = blockEncrypt(&cipher, &key, PT, BLOCK_LEN, CT);
            if( outLen!=BLOCK_LEN ) {
                printf("Error in ecb_vt\n");
                exit(1);
            }

            /* print out i, UK and CT(final value) */
            fprintf(fp1, "\n\nI=%d", i+1);
            BytePrint(fp1, "\nPT", BLOCK_SIZE, PT);
            BytePrint(fp1, "\nCT", BLOCK_SIZE, CT);

            /* print out i, UK, CT and intermediate values */
            fprintf(fp2, "\n\nI=%d", i+1);
            BytePrint(fp2, "\nPT", BLOCK_SIZE, PT);
            BytePrint(fp2, "\nCT", BLOCK_SIZE, CT);
            for( k=0; k<=NoRounds; k++) {
                IM[5] = k/10 + '0';
                IM[6] = k%10 + '0';
                BytePrint(fp2, IM, BLOCK_SIZE, pdwInterVal[k]);
            }

            /* test that encryption and decryption are correct */
            makeKey(&key, DIR_DECRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);
            outLen = blockDecrypt(&cipher, &key, CT, BLOCK_LEN, DT);
            if( (outLen!=BLOCK_LEN) || (memcmp(PT, DT, BLOCK_SIZE)!=0) ) {
                printf("Error in ecb_vt\n");
                exit(1);
            }
        }
        fprintf(fp1, "\n\n==========\n");
        fprintf(fp2, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp1);
    fclose(fp2);
}

/***********************************************************************
* 8x8 S-boxes S0, S1 : usage check.
*/
void UpdateTable(pdVal, i, FLAG)
DWORD *pdVal;
int i, FLAG;
{
    if( FLAG==0 ) {
        Table_S0[i][(pdVal[0]    ) & 0xFF]++;
        Table_S1[i][(pdVal[0]>> 8) & 0xFF]++;
        Table_S0[i][(pdVal[0]>>16) & 0xFF]++;
        Table_S1[i][(pdVal[0]>>24) & 0xFF]++;
        Table_S1[i][(pdVal[1]    ) & 0xFF]++;
        Table_S0[i][(pdVal[1]>> 8) & 0xFF]++;
        Table_S1[i][(pdVal[1]>>16) & 0xFF]++;
        Table_S0[i][(pdVal[1]>>24) & 0xFF]++;
        Table_S0[i][(pdVal[2]    ) & 0xFF]++;
        Table_S1[i][(pdVal[2]>> 8) & 0xFF]++;
        Table_S0[i][(pdVal[2]>>16) & 0xFF]++;
        Table_S1[i][(pdVal[2]>>24) & 0xFF]++;
        Table_S1[i][(pdVal[3]    ) & 0xFF]++;
        Table_S0[i][(pdVal[3]>> 8) & 0xFF]++;
        Table_S1[i][(pdVal[3]>>16) & 0xFF]++;
        Table_S0[i][(pdVal[3]>>24) & 0xFF]++;
    }
    else {
        Table_S1[i][(pdVal[0]    ) & 0xFF]++;
        Table_S0[i][(pdVal[0]>> 8) & 0xFF]++;
        Table_S1[i][(pdVal[0]>>16) & 0xFF]++;
        Table_S0[i][(pdVal[0]>>24) & 0xFF]++;
        Table_S0[i][(pdVal[1]    ) & 0xFF]++;
        Table_S1[i][(pdVal[1]>> 8) & 0xFF]++;
        Table_S0[i][(pdVal[1]>>16) & 0xFF]++;
        Table_S1[i][(pdVal[1]>>24) & 0xFF]++;
        Table_S1[i][(pdVal[2]    ) & 0xFF]++;
        Table_S0[i][(pdVal[2]>> 8) & 0xFF]++;
        Table_S1[i][(pdVal[2]>>16) & 0xFF]++;
        Table_S0[i][(pdVal[2]>>24) & 0xFF]++;
        Table_S0[i][(pdVal[3]    ) & 0xFF]++;
        Table_S1[i][(pdVal[3]>> 8) & 0xFF]++;
        Table_S0[i][(pdVal[3]>>16) & 0xFF]++;
        Table_S1[i][(pdVal[3]>>24) & 0xFF]++;
    }
}

/*************************** Tables KAT in ECB mode ***********************/
void ecb_tbl()
{
    FILE *fp;
    int i=16, j, k, KeyLen, outLen;
    BYTE UK[MaxIter4IntVal][MAX_KEY_SIZE],      /* User Key */
         PT[MaxIter4IntVal][BLOCK_SIZE], CT[MaxIter4IntVal][BLOCK_SIZE];
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("ecb_tbl.txt...\n");
    fp = fopen("ecb_tbl.txt", "w");
    FileHead(fp, "ecb_tbl.txt", "Electronic Codebook (ECB) Mode",
                    "Tables Known Answer Test");
    srand(i);

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        fprintf(fp, "\nKEYSIZE=%d", KeyLen);
        for( i=0; i<256; i++)   /* Set Table_S0[0], Table_S1[0] by 0 */
            Table_S0[0][i] = Table_S1[0][i] = 0;

        for( i=1; i<MaxIter4IntVal-2; i++) {
            /* Get random UK and PT and encrypt */
            for( j=0; j<KeyLen/8; j++)      UK[i][j] = (BYTE)rand();
            for( j=0; j<BLOCK_LEN/8; j++)   PT[i][j] = (BYTE)rand();
            makeKey(&key, DIR_ENCRYPT, KeyLen, UK[i]);
            cipherInit(&cipher, MODE_ECB, NULL);
            outLen = blockEncrypt(&cipher, &key, PT[i], BLOCK_LEN, CT[i]);
            if( outLen!=BLOCK_LEN ) {
                printf("Error in ecb_tbl\n");
                exit(1);
            }

            /* Set Table_S0[i], Table_S1[i] by 0 and update */
            for( j=0; j<256; j++)   Table_S0[i][j] = Table_S1[i][j] = 0;
            for( j=0; j<12; j++)    UpdateTable(pdwInterVal[j], i, j&1);

            /* if no use of new S-Box, continue */
            for( j=0; j<256; j++) {
                if( (Table_S0[0][j]==0) && (Table_S0[i][j]!=0) )    break;
                if( (Table_S1[0][j]==0) && (Table_S1[i][j]!=0) )    break;
            }
            if( j==256 ) {
                i--;
                continue;
            }

            /* Update Table_S0[i], Table_S1[i] by 0 */
            for( j=0; j<256; j++) {
                Table_S0[0][j] += Table_S0[i][j];
                Table_S1[0][j] += Table_S1[i][j];
            }

            /* if all S-Box is used at least once, break */
            for( j=0; j<256; j++) {
                if( (Table_S0[0][j]==0) || (Table_S1[0][j]==0) )
                    break;
            }
            if( j==256 )
                break;
        }

        if( i==MaxIter4IntVal-2 ) {     /* error. Can't find */
            for( j=0; j<256; j++) {
                if( j%16==0 ) fprintf(fp, "\n");
                fprintf(fp, "%d/%d ", Table_S0[0][j], Table_S1[0][j]);
            }
            fprintf(fp, "\n");
        }
        else {      /* print out the result */
            k = i;
            for( i=1; i<=k; i++) {
                fprintf(fp, "\n\nI=%d", i);
                BytePrint(fp, "\nKEY", KeyLen/8, UK[i]);
                BytePrint(fp, "\nPT", BLOCK_SIZE, PT[i]);
                BytePrint(fp, "\nCT", BLOCK_SIZE, CT[i]);
            }
        }
        fprintf(fp, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp);
}


/************************* MCT in ECB Encryption mode ***********************/
void ecb_e_m()
{
    FILE *fp;
    int i, j, KeyLen, outLen;
    BYTE UK[MAX_KEY_SIZE], PT[BLOCK_SIZE], CT[BLOCK_SIZE];
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("ecb_e_m.txt...\n");
    fp = fopen("ecb_e_m.txt", "w");
    FileHead(fp, "ecb_e_m.txt", "Electronic Codebook (ECB) Mode - ENCRYPTION",
                    "Monte Carlo Test");

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        /* Set UK, CT by 0 */
        for(i=0; i<KeyLen/8; ++i)       UK[i] = 0;
        for(i=0; i<BLOCK_SIZE; ++i)     CT[i] = 0;
        fprintf(fp, "\nKEYSIZE=%d", KeyLen);

        /* Outer loop */
        for( i=0; i<400; i++) {
            /* print out */
            fprintf(fp, "\n\nI=%d", i);
            BytePrint(fp, "\nKEY", KeyLen/8, UK);
            BytePrint(fp, "\nPT", BLOCK_SIZE, CT);

            /* encrypt */
            makeKey(&key, DIR_ENCRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);

            /* Inner loop */
            for( j=0; j<MCT_ITER; j++) {
                BlockCopy(PT, CT);
                outLen = blockEncrypt(&cipher, &key, PT, BLOCK_LEN, CT);
                if( outLen!=BLOCK_LEN ) {
                    printf("Error in ecb_e_m\n");
                    exit(1);
                }
            }

            /* print out */
            BytePrint(fp, "\nCT", BLOCK_SIZE, CT);

            /* Update UK for next step */
            if( KeyLen==128 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= CT[j];
            }
            else if( KeyLen==256 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= PT[j];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE] ^= CT[j];
            }
            else if( KeyLen==192 ) {
                for( j=0; j<BLOCK_SIZE/2; j++)
                    UK[j] ^= PT[j+BLOCK_SIZE/2];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE/2] ^= CT[j];
            }
        }
        fprintf(fp, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp);
}

/************************ MCT in ECB Decryption mode ************************/
void ecb_d_m()
{
    FILE *fp;
    int i, j, KeyLen, outLen;
    BYTE UK[MAX_KEY_SIZE], PT[BLOCK_SIZE], CT[BLOCK_SIZE];
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("ecb_d_m.txt...\n");
    fp = fopen("ecb_d_m.txt", "w");
    FileHead(fp, "ecb_d_m.txt", "Electronic Codebook (ECB) Mode - DECRYPTION",
                    "Monte Carlo Test");

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        /* Set UK, CT by 0 */
        for(i=0; i<KeyLen/8; ++i)       UK[i] = 0;
        for(i=0; i<BLOCK_SIZE; ++i)     CT[i] = 0;
        fprintf(fp, "\nKEYSIZE=%d", KeyLen);

        /* Outer loop */
        for( i=0; i<400; i++) {
            /* print out */
            fprintf(fp, "\n\nI=%d", i);
            BytePrint(fp, "\nKEY", KeyLen/8, UK);
            BytePrint(fp, "\nCT", BLOCK_SIZE, CT);

            /* encrypt */
            makeKey(&key, DIR_DECRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);

            /* Inner loop */
            for( j=0; j<MCT_ITER; j++) {
                BlockCopy(PT, CT);
                outLen = blockDecrypt(&cipher, &key, PT, BLOCK_LEN, CT);
                if( outLen!=BLOCK_LEN ) {
                    printf("Error in ecb_d_m\n");
                    exit(1);
                }
            }

            /* print out */
            BytePrint(fp, "\nPT", BLOCK_SIZE, CT);

            /* Update UK for next step */
            if( KeyLen==128 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= CT[j];
            }
            else if( KeyLen==256 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= PT[j];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE] ^= CT[j];
            }
            else if( KeyLen==192 ) {
                for( j=0; j<BLOCK_SIZE/2; j++)
                    UK[j] ^= PT[j+BLOCK_SIZE/2];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE/2] ^= CT[j];
            }
        }
        fprintf(fp, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp);
}

/*********************** MCT in CBC Encryption mode ************************/
void cbc_e_m()
{
    FILE *fp;
    int i, j, KeyLen, outLen;
    BYTE UK[MAX_KEY_SIZE], PT[BLOCK_SIZE], CT[BLOCK_SIZE],
         IV[BLOCK_SIZE], IB[BLOCK_SIZE];
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("cbc_e_m.txt...\n");
    fp = fopen("cbc_e_m.txt", "w");
    FileHead(fp, "cbc_e_m.txt", "Cipher Block Chaining (CBC) Mode - ENCRYPTION",
                    "Monte Carlo Test");

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        /* Set UK, CT and IV by 0 */
        for(i=0; i<KeyLen/8; ++i)       UK[i] = 0;
        for(i=0; i<BLOCK_SIZE; ++i)     PT[i] = IV[i] = 0;
        fprintf(fp, "\nKEYSIZE=%d", KeyLen);

        /* Outer loop */
        for( i=0; i<400; i++) {
            /* print out */
            fprintf(fp, "\n\nI=%d", i);
            BytePrint(fp, "\nKEY", KeyLen/8, UK);
            BytePrint(fp, "\nIV", BLOCK_SIZE, IV);
            BytePrint(fp, "\nPT", BLOCK_SIZE, PT);

            /* encrypt */
            makeKey(&key, DIR_ENCRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);
            BlockCopy(CT, IV);

            /* Inner loop */
            for( j=0; j<MCT_ITER; j++) {
                BlockXor(IB, PT, IV);
                BlockCopy(PT, CT);
                outLen = blockEncrypt(&cipher, &key, IB, BLOCK_LEN, CT);
                if( outLen!=BLOCK_LEN ) {
                    printf("Error in cbc_e_m\n");
                    exit(1);
                }
                BlockCopy(IV, CT);
            }

            /* print out */
            BytePrint(fp, "\nCT", BLOCK_SIZE, CT);

            /* Update UK for next step */
            if( KeyLen==128 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= CT[j];
            }
            else if( KeyLen==256 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= PT[j];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE] ^= CT[j];
            }
            else if( KeyLen==192 ) {
                for( j=0; j<BLOCK_SIZE/2; j++)
                    UK[j] ^= PT[j+BLOCK_SIZE/2];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE/2] ^= CT[j];
            }
        }
        fprintf(fp, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp);
}

/************************ MCT in CBC Decryption mode ***********************/
void cbc_d_m()
{
    FILE *fp;
    int i, j, KeyLen, outLen;
    BYTE UK[MAX_KEY_SIZE], PT[BLOCK_SIZE], CT[BLOCK_SIZE], IV[BLOCK_SIZE];
    keyInstance key;
    cipherInstance cipher;

    /* Init part */
    printf("cbc_d_m.txt...\n");
    fp = fopen("cbc_d_m.txt", "w");
    FileHead(fp, "cbc_d_m.txt", "Cipher Block Chaining (CBC) Mode - DECRYPTION",
                    "Monte Carlo Test");

    /* main part */
    for( KeyLen=128; KeyLen<=256; KeyLen+=64) {
        /* Set UK, CT and IV by 0 */
        for(i=0; i<KeyLen/8; ++i)       UK[i] = 0;
        for(i=0; i<BLOCK_SIZE; ++i)     CT[i] = IV[i] = 0;
        fprintf(fp, "\nKEYSIZE=%d", KeyLen);

        /* Outer loop */
        for( i=0; i<400; i++) {
            /* print out */
            fprintf(fp, "\n\nI=%d", i);
            BytePrint(fp, "\nKEY", KeyLen/8, UK);
            BytePrint(fp, "\nIV", BLOCK_SIZE, IV);
            BytePrint(fp, "\nCT", BLOCK_SIZE, CT);

            /* encrypt */
            makeKey(&key, DIR_DECRYPT, KeyLen, UK);
            cipherInit(&cipher, MODE_ECB, NULL);

            /* Inner loop */
            for( j=0; j<MCT_ITER; j++) {
                outLen = blockDecrypt(&cipher, &key, CT, BLOCK_LEN, PT);
                if( outLen!=BLOCK_LEN ) {
                    printf("Error in cbc_e_m\n");
                    exit(1);
                }
                BlockXor(PT, PT, IV);
                BlockCopy(IV, CT);
                BlockCopy(CT, PT);
            }

            /* print out */
            BytePrint(fp, "\nPT", BLOCK_SIZE, CT);

            /* Update UK for next step */
            if( KeyLen==128 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= CT[j];
            }
            else if( KeyLen==256 ) {
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j] ^= IV[j];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE] ^= CT[j];
            }
            else if( KeyLen==192 ) {
                for( j=0; j<BLOCK_SIZE/2; j++)
                    UK[j] ^= IV[j+BLOCK_SIZE/2];
                for( j=0; j<BLOCK_SIZE; j++)
                    UK[j+BLOCK_SIZE/2] ^= CT[j];
            }
        }
        fprintf(fp, "\n\n==========\n");
    }

    /* Final part */
    fclose(fp);
}

/******************************************************************************/
