/*******************************************************************************
*
* FILE:         cryptonsbox.c
*
* DESCRIPTION:    main program to generate S-boxes for Crypton Version 0.5
*
* AUTHOR:        Chae Hoon Lim and Hyo Sun Hwang
*                E-mail: chlim@future.co.kr, hyosun@future.co.kr
*                http://crypt.future.co.kr/~chlim
*                Information and Communications Research Center,
*                Future Systems, Inc.
*                372-2, Yang Jae-Dong, Seo Cho-Gu, Seoul, 137-130, KOREA
*                Tel: +82-2-578-0581 (ext. 557), Fax: +82-2-578-0584
*
* DATE:         July 10, 1998
*
* NOTE: This code is used to generate and file-print two 8 x 8 S-boxes,
*       S0 and S1, and four extended S-boxes, SS0 - SS3. Just run it once 
*       at the installation time to generate the include file "crypton.tab". 
**
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

typedef unsigned char           BYTE;

void main(void)
{
    int i, k, R, L;
    static BYTE m[8] = { 0xfc, 0xf3, 0xcf, 0x3f, 0xfc, 0xf3, 0xcf, 0x3f};
    BYTE S[2][256];
    static BYTE P0[16] = {15, 9, 6, 8, 9, 9, 4,12, 6, 2, 6,10, 1, 3, 5,15};
    static BYTE P1[16] = {10,15, 4, 7, 5, 2,14, 6, 9, 3,12, 8,13, 1,11, 0};
    static BYTE P2[16] = { 0, 4, 8, 4, 2,15, 8,13, 1, 1,15, 7, 2,11,14,15};
    FILE *fp;

    /* generate two S-boxes from P0, P1, P2 using a 3-round Feistel cipher */
    for(i=0; i<256; ++i) {
        R = (i & 0x0F);
        L = (i>>4) ^ P0[R];
        R ^= P1[L];
        L ^= P2[R];
        R ^= (L<< 4);

        S[0][i] = (BYTE)R;
        S[1][R] = (BYTE)i;
    }
    fp = fopen("crypton.tab", "w");

    /* file print S-boxes */
    for(k=0; k<2; ++k) {
        fprintf(fp, "static BYTE S%d[256] = {\n", k);
        for(i=0; i<256; i++) {
            fprintf(fp, "0x%02x", S[k][i]);
            if( i==255 )            fprintf(fp, "\n};\n\n");
            else if( (i%8)==7 )    fprintf(fp, ",\n");
            else                    fprintf(fp, " ,");
        }
    }

    /* file print SS-boxes */
    fprintf(fp, "\n#ifdef BIG_ENDIAN\n\n"); /* for BIG_ENDIAN machines */

    for(k=0; k<4; ++k) {
        fprintf(fp, "static DWORD SS%d[256] = {\n", k);
        for(i=0; i<256; i++) {
            L = S[k&1][i];
            fprintf(fp, " 0x%02x%02x%02x%02x",
                        L&m[k], L&m[k+1], L&m[k+2], L&m[k+3]);
            if( i==255 )            fprintf(fp, "\n};\n\n");
            else if( (i%4)==3 )     fprintf(fp, ",\n");
            else                    fprintf(fp, ",");
        }
    }

    fprintf(fp, "#else\n\n");  /* for LITTLE_ENDIAN machines */
    for(k=0; k<4; ++k) {
        fprintf(fp, "static DWORD SS%d[256] = {\n", k);
        for(i=0; i<256; i++) {
            L = S[k&1][i];
            fprintf(fp, " 0x%02x%02x%02x%02x",
                        L&m[k+3], L&m[k+2], L&m[k+1], L&m[k]);
            if( i==255 )            fprintf(fp, "\n};\n\n");
            else if( (i%4)==3 )        fprintf(fp, ",\n");
            else                    fprintf(fp, ",");
        }
    }
    fprintf(fp, "#endif\n");

    fclose(fp);
}
